from __future__ import annotations
from typing import Any
from ...thoth_json_core.types import IDecoderHelpers_1
from ...thoth_json_python.decode import Decode_helpers

helpers: IDecoderHelpers_1[Any] = Decode_helpers

__all__ = ["helpers"]

